@@ -2,7 +2,7 @@
  * Copyright (C) 2008 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 3 as 
+ * it under the terms of the GNU General Public License version 3 as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -25,16 +25,13 @@
 
 #include <math.h>
 
-#include "task-list.h"
-
 G_DEFINE_TYPE (TaskTitle, task_title, GTK_TYPE_EVENT_BOX);
 
 #define TASK_TITLE_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj),\
   TASK_TYPE_TITLE, \
   TaskTitlePrivate))
 
-#define LOGOUT "mate-session-save --kill --gui"
-#define SHOW_HOME_TITLE_KEY "show-home-title"
+#define LOGOUT "mate-session-save --logout-dialog"
 
 struct _TaskTitlePrivate
 {
@@ -48,24 +45,75 @@ struct _TaskTitlePrivate
   GdkPixbuf *quit_icon;
 
   gboolean show_home_title;
+  gboolean bold_window_title;
   gboolean mouse_in_close_button;
 };
 
+enum
+{
+  PROP_0,
+  PROP_SHOW_HOME_TITLE,
+  PROP_BOLD_WINDOW_TITLE
+};
+
 static void disconnect_window (TaskTitle *title);
 
 static gboolean
-on_close_clicked (GtkButton *button, 
-                  GdkEventButton *event, 
+start_logout_dialog (TaskTitle *title)
+{
+  GError *error = NULL;
+  GAppInfo *app_info;
+  GdkScreen *gdkscreen;
+
+  g_return_val_if_fail (TASK_IS_TITLE (title), FALSE);
+
+  gdkscreen = gtk_widget_get_screen (GTK_WIDGET (title));
+  app_info = g_app_info_create_from_commandline (
+    LOGOUT, _("Log off, switch user, lock screen or power down the computer"),
+    G_APP_INFO_CREATE_NONE, &error);
+
+  if (!error) {
+    GdkDisplay *display;
+    GdkAppLaunchContext *launch_context;
+    display = gdk_screen_get_display (gdkscreen);
+    launch_context = gdk_display_get_app_launch_context (display);
+    gdk_app_launch_context_set_screen (launch_context, gdkscreen);
+    g_app_info_launch (app_info, NULL, G_APP_LAUNCH_CONTEXT (launch_context), &error);
+    g_object_unref (launch_context);
+    return TRUE;
+  }
+
+  GtkWidget *dialog;
+  dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_DESTROY_WITH_PARENT,
+                                   GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
+                                   _("There was an error executing '%s': %s"),
+                                   LOGOUT, error->message);
+
+  g_signal_connect (dialog, "response", G_CALLBACK (gtk_widget_destroy), NULL);
+
+  gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+  gtk_window_set_screen (GTK_WINDOW (dialog), gdkscreen);
+
+  gtk_widget_show (dialog);
+  g_error_free (error);
+  return FALSE;
+}
+
+static gboolean
+on_close_clicked (GtkButton *button,
+                  GdkEventButton *event,
                   TaskTitle *title)
 {
   TaskTitlePrivate *priv;
   WnckWindow *window;
+  gboolean retval;
 
   g_return_val_if_fail (TASK_IS_TITLE (title), FALSE);
   priv = title->priv;
+  retval = FALSE;
 
   if (event->button != 1 || !priv->mouse_in_close_button)
-    return FALSE;
+    return retval;
 
   window = wnck_screen_get_active_window (priv->screen);
 
@@ -74,11 +122,22 @@ on_close_clicked (GtkButton *button,
   {
     if (priv->window == window)
       disconnect_window (title);
-    wnck_window_close (window, event->time);
+    // event->time is buggy, thus we have to workaroud this
+    GdkScreen *gdkscreen;
+    GdkDisplay *display;
+    gdkscreen = gtk_widget_get_screen (GTK_WIDGET (title));
+    display = gdk_screen_get_display (gdkscreen);
+    wnck_window_close (window, gdk_x11_display_get_user_time (display));
+    retval = TRUE;
+  }
+  else if (priv->show_home_title)
+  {
+    // This is a logout click
+    retval = start_logout_dialog (title);
   }
   gtk_widget_queue_draw (GTK_WIDGET (title));
 
-  return TRUE;
+  return retval;
 }
 
 static gboolean
@@ -87,10 +146,10 @@ on_enter_notify (GtkWidget *widget,
                  TaskTitle *title)
 {
   g_return_val_if_fail (TASK_IS_TITLE (title), FALSE);
-  
+
   title->priv->mouse_in_close_button = TRUE;
   gtk_widget_queue_draw (widget);
-  
+
   return FALSE;
 }
 
@@ -100,7 +159,7 @@ on_leave_notify (GtkWidget *widget,
                  TaskTitle *title)
 {
   g_return_val_if_fail (TASK_IS_TITLE (title), FALSE);
-  
+
   title->priv->mouse_in_close_button = FALSE;
   gtk_widget_queue_draw (widget);
 
@@ -108,7 +167,7 @@ on_leave_notify (GtkWidget *widget,
 }
 
 static gboolean
-on_button_draw (GtkWidget *widget, 
+on_button_draw (GtkWidget *widget,
                 cairo_t *cr,
                 TaskTitle *title)
 {
@@ -116,7 +175,7 @@ on_button_draw (GtkWidget *widget,
 
   TaskTitlePrivate *priv;
   priv = title->priv;
-  
+
   if (priv->mouse_in_close_button)
   {
     GtkStyle *style = gtk_widget_get_style (widget);
@@ -194,14 +253,16 @@ on_state_changed (WnckWindow *window,
 
   if (wnck_window_is_maximized (window))
   {
-    gtk_widget_set_state (GTK_WIDGET (title), GTK_STATE_ACTIVE);
+    gtk_widget_set_state_flags (GTK_WIDGET (title), GTK_STATE_FLAG_ACTIVE,
+                                TRUE);
     gtk_widget_show (priv->box);
   }
   else
   {
-    gtk_widget_set_state (GTK_WIDGET (title), GTK_STATE_NORMAL);
+    gtk_widget_set_state_flags (GTK_WIDGET (title), GTK_STATE_FLAG_NORMAL,
+                               TRUE);
     gtk_widget_hide (priv->box);
-  }  
+  }
 }
 
 static void
@@ -224,7 +285,7 @@ on_active_window_changed (WnckScreen *screen,
   TaskTitlePrivate *priv;
   WnckWindow *act_window;
   WnckWindowType type = WNCK_WINDOW_NORMAL;
-  
+
   g_return_if_fail (TASK_IS_TITLE (title));
   priv = title->priv;
 
@@ -241,34 +302,39 @@ on_active_window_changed (WnckScreen *screen,
       || type == WNCK_WINDOW_SPLASHSCREEN
       || type == WNCK_WINDOW_MENU)
     return;
- 
+
   disconnect_window (title);
 
   if (!WNCK_IS_WINDOW (act_window)
         || wnck_window_get_window_type (act_window) == WNCK_WINDOW_DESKTOP)
-  { 
+  {
     if (priv->show_home_title)
     {
-      gtk_label_set_text (GTK_LABEL (priv->label), _("Home"));
-      gtk_image_set_from_pixbuf (GTK_IMAGE (priv->button_image), 
+      gtk_label_set_text (GTK_LABEL (priv->label), _("Desktop"));
+      gtk_image_set_from_pixbuf (GTK_IMAGE (priv->button_image),
                                  priv->quit_icon);
+
+      gtk_widget_set_tooltip_text (GTK_WIDGET (title), NULL);
       gtk_widget_set_tooltip_text (priv->button,
-                                _("Log off, switch user, lock screen or power "
-                                     "down the computer"));
-      gtk_widget_set_tooltip_text (GTK_WIDGET (title),
-                                   _("Home"));      gtk_widget_show (priv->box);
+                                   _("Log off, switch user, lock screen or "
+                                     "power down the computer"));
+
+      gtk_widget_set_state_flags (GTK_WIDGET (title), GTK_STATE_FLAG_ACTIVE,
+                                  TRUE);
+      gtk_widget_show (priv->box);
     }
     else
     {
-      gtk_widget_set_state (GTK_WIDGET (title), GTK_STATE_NORMAL);
       gtk_widget_set_tooltip_text (priv->button, NULL);
       gtk_widget_set_tooltip_text (GTK_WIDGET (title), NULL);
+      gtk_widget_set_state_flags (GTK_WIDGET (title), GTK_STATE_FLAG_NORMAL,
+                                  TRUE);
       gtk_widget_hide (priv->box);
     }
   }
   else
   {
-    gtk_label_set_text (GTK_LABEL (priv->label), 
+    gtk_label_set_text (GTK_LABEL (priv->label),
                         wnck_window_get_name (act_window));
     gtk_image_set_from_icon_name (GTK_IMAGE (priv->button_image),
                                   "window-close", GTK_ICON_SIZE_MENU);
@@ -283,42 +349,20 @@ on_active_window_changed (WnckScreen *screen,
                       G_CALLBACK (on_icon_changed), title);
     g_signal_connect_after (act_window, "state-changed",
                             G_CALLBACK (on_state_changed), title);
-    gtk_widget_show (priv->box);  
     priv->window = act_window;
-  }
-
-  if (WNCK_IS_WINDOW (act_window)
-      && !wnck_window_is_maximized (act_window)
-      && (priv->show_home_title ? type != WNCK_WINDOW_DESKTOP : 1))
-  {
-    gtk_widget_set_state (GTK_WIDGET (title), GTK_STATE_NORMAL);
-    gtk_widget_hide (priv->box);
-  }
-  else if (!WNCK_IS_WINDOW (act_window))
-  {
-    if (task_list_get_desktop_visible (TASK_LIST (task_list_get_default ()))
-        && priv->show_home_title)
+    if (wnck_window_is_maximized (act_window))
     {
-      gtk_label_set_text (GTK_LABEL (priv->label), _("Home"));
-      gtk_image_set_from_pixbuf (GTK_IMAGE (priv->button_image), 
-                                 priv->quit_icon);
-      gtk_widget_set_tooltip_text (priv->button,
-                                _("Log off, switch user, lock screen or power "
-                                     "down the computer"));
-      gtk_widget_set_tooltip_text (GTK_WIDGET (title),
-                                   _("Home"));      
+      gtk_widget_set_state_flags (GTK_WIDGET (title), GTK_STATE_FLAG_ACTIVE,
+                                  TRUE);
       gtk_widget_show (priv->box);
-     }
+    }
     else
-    {  
-      gtk_widget_set_state (GTK_WIDGET (title), GTK_STATE_NORMAL);
-      gtk_widget_set_tooltip_text (priv->button, NULL);
-      gtk_widget_set_tooltip_text (GTK_WIDGET (title), NULL);
+    {
+      gtk_widget_set_state_flags (GTK_WIDGET (title), GTK_STATE_FLAG_NORMAL,
+                                  TRUE);
       gtk_widget_hide (priv->box);
     }
   }
-  else
-    gtk_widget_set_state (GTK_WIDGET (title), GTK_STATE_ACTIVE);
 
   gtk_widget_queue_draw (GTK_WIDGET (title));
 }
@@ -328,7 +372,6 @@ on_button_release (GtkWidget *title, GdkEventButton *event)
 {
   TaskTitlePrivate *priv;
   WnckWindow *window;
-  GtkWidget *menu;
 
   g_return_val_if_fail (TASK_IS_TITLE (title), FALSE);
   priv = TASK_TITLE_GET_PRIVATE (title);
@@ -341,9 +384,9 @@ on_button_release (GtkWidget *title, GdkEventButton *event)
   {
     if (wnck_window_get_window_type (window) != WNCK_WINDOW_DESKTOP)
     {
+      GtkWidget *menu;
       menu = wnck_action_menu_new (window);
-      gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 
-                      event->button, event->time);
+      gtk_menu_popup_at_pointer (GTK_MENU (menu), (GdkEvent*)event);
       return TRUE;
     }
   }
@@ -354,7 +397,7 @@ on_button_release (GtkWidget *title, GdkEventButton *event)
       wnck_window_unmaximize (window);
     }
   }
-  
+
   return FALSE;
 }
 
@@ -377,6 +420,89 @@ on_draw (GtkWidget *w, cairo_t *cr)
 }
 
 /* GObject stuff */
+static void
+task_title_set_show_home_title (TaskTitle *title, gboolean show_home_title)
+{
+  TaskTitlePrivate *priv = title->priv;
+
+  priv->show_home_title = show_home_title;
+
+  g_debug ("Show home title: %s", show_home_title ? "true" : "false");
+
+  on_active_window_changed(priv->screen, priv->window, title);
+}
+
+static void
+task_title_set_bold_window_title (TaskTitle *title, gboolean bold_window_title)
+{
+  TaskTitlePrivate *priv = title->priv;
+
+  priv->bold_window_title = bold_window_title;
+
+  g_debug ("Bold window title: %s", bold_window_title ? "true" : "false");
+
+  PangoAttribute *attr;
+  PangoAttrList *attr_list = pango_attr_list_new ();
+  if (priv->bold_window_title)
+  {
+    attr = pango_attr_weight_new (PANGO_WEIGHT_BOLD);
+  }
+  else
+  {
+    attr = pango_attr_weight_new (PANGO_WEIGHT_NORMAL);
+  }
+  pango_attr_list_insert (attr_list, attr);
+  gtk_label_set_attributes (GTK_LABEL (priv->label), attr_list);
+}
+
+static void
+task_title_get_property (GObject    *object,
+                         guint       prop_id,
+                         GValue     *value,
+                         GParamSpec *pspec)
+{
+  TaskTitle *title = TASK_TITLE (object);
+  TaskTitlePrivate *priv;
+
+  g_return_if_fail (TASK_IS_TITLE (title));
+  priv = title->priv;
+
+  switch (prop_id)
+  {
+    case PROP_SHOW_HOME_TITLE:
+      g_value_set_boolean (value, priv->show_home_title);
+      break;
+    case PROP_BOLD_WINDOW_TITLE:
+      g_value_set_boolean (value, priv->bold_window_title);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+  }
+}
+
+static void
+task_title_set_property (GObject      *object,
+                         guint         prop_id,
+                         const GValue *value,
+                         GParamSpec   *pspec)
+{
+  TaskTitle *title = TASK_TITLE (object);
+
+  g_return_if_fail (TASK_IS_TITLE (title));
+
+  switch (prop_id)
+  {
+    case PROP_SHOW_HOME_TITLE:
+      task_title_set_show_home_title (title, g_value_get_boolean (value));
+      break;
+    case PROP_BOLD_WINDOW_TITLE:
+      task_title_set_bold_window_title (title, g_value_get_boolean (value));
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+  }
+}
+
 static void
 task_title_finalize (GObject *object)
 {
@@ -397,6 +523,25 @@ task_title_class_init (TaskTitleClass *klass)
   GtkWidgetClass      *wid_class = GTK_WIDGET_CLASS (klass);
 
   obj_class->finalize = task_title_finalize;
+  obj_class->set_property = task_title_set_property;
+  obj_class->get_property = task_title_get_property;
+
+  g_object_class_install_property (obj_class,
+    PROP_SHOW_HOME_TITLE,
+    g_param_spec_boolean ("show_home_title",
+                          "Show Home Title",
+                          "Show Home Title",
+                          FALSE,
+                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+
+  g_object_class_install_property (obj_class,
+    PROP_BOLD_WINDOW_TITLE,
+    g_param_spec_boolean ("bold_window_title",
+                          "Bold Window Title",
+                          "Show windows title with a bold face",
+                          TRUE,
+                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+
   wid_class->draw = on_draw;
 
   g_type_class_add_private (obj_class, sizeof (TaskTitlePrivate));
@@ -408,23 +553,22 @@ task_title_init (TaskTitle *title)
   TaskTitlePrivate *priv;
   GdkScreen *gdkscreen;
   GtkIconTheme *theme;
-  GdkPixbuf *pixbuf;
   AtkObject *atk;
   int width, height;
-    	
+
   priv = title->priv = TASK_TITLE_GET_PRIVATE (title);
 
   priv->screen = wnck_screen_get_default ();
   priv->window = NULL;
 
-  /* FIXME we can add an option for this in future */
-  /* now it's disabled with gsettings migration */
+  // Default values
   priv->show_home_title = FALSE;
+  priv->bold_window_title = TRUE;
 
   gtk_widget_add_events (GTK_WIDGET (title), GDK_ALL_EVENTS_MASK);
 
   priv->align = gtk_alignment_new (0.0, 0.5, 1.0, 1.0);
-  gtk_alignment_set_padding (GTK_ALIGNMENT (priv->align), 
+  gtk_alignment_set_padding (GTK_ALIGNMENT (priv->align),
                              0, 0, 6, 6);
   gtk_container_add (GTK_CONTAINER (title), priv->align);
 
@@ -433,32 +577,35 @@ task_title_init (TaskTitle *title)
   gtk_widget_set_no_show_all (priv->box, TRUE);
   gtk_widget_show (priv->box);
 
-  priv->label = gtk_label_new (_("Home"));
+  priv->label = gtk_label_new (_("Desktop"));
   gtk_label_set_ellipsize (GTK_LABEL (priv->label), PANGO_ELLIPSIZE_END);
   gtk_widget_set_halign (priv->label, GTK_ALIGN_START);
   gtk_widget_set_valign (priv->label, GTK_ALIGN_CENTER);
-  
-  PangoAttrList *attr_list = pango_attr_list_new ();
-  PangoAttribute *attr = pango_attr_weight_new (PANGO_WEIGHT_BOLD);
-  pango_attr_list_insert (attr_list, attr);
-  gtk_label_set_attributes (GTK_LABEL (priv->label), attr_list);
-  
+
+  if (priv->bold_window_title)
+  {
+    PangoAttrList *attr_list = pango_attr_list_new ();
+    PangoAttribute *attr = pango_attr_weight_new (PANGO_WEIGHT_BOLD);
+    pango_attr_list_insert (attr_list, attr);
+    gtk_label_set_attributes (GTK_LABEL (priv->label), attr_list);
+  }
+
   gtk_box_pack_start (GTK_BOX (priv->box), priv->label, TRUE, TRUE, 0);
   gtk_widget_show (priv->label);
 
-  priv->button = g_object_new (GTK_TYPE_EVENT_BOX, 
+  priv->button = g_object_new (GTK_TYPE_EVENT_BOX,
                                "visible-window", FALSE,
                                "above-child", TRUE,
                                NULL);
   gtk_box_pack_start (GTK_BOX (priv->box), priv->button, FALSE, FALSE, 0);
   gtk_widget_show (priv->button);
-  
+
   atk = gtk_widget_get_accessible (priv->button);
   atk_object_set_name (atk, _("Close"));
   atk_object_set_description (atk, _("Close current window."));
   atk_object_set_role (atk, ATK_ROLE_PUSH_BUTTON);
-  
-  g_signal_connect (priv->button, "button-release-event", 
+
+  g_signal_connect (priv->button, "button-release-event",
                     G_CALLBACK (on_close_clicked), title);
   g_signal_connect (priv->button, "enter-notify-event",
                     G_CALLBACK (on_enter_notify), title);
@@ -467,30 +614,20 @@ task_title_init (TaskTitle *title)
   g_signal_connect (priv->button, "draw",
                     G_CALLBACK (on_button_draw), title);
 
+  // Initialize button_image to allow easy setting by icon name after that.
   gdkscreen = gtk_widget_get_screen (GTK_WIDGET (title));
   theme = gtk_icon_theme_get_for_screen (gdkscreen);
-  gtk_icon_size_lookup (GTK_ICON_SIZE_MENU,
-                        &width, &height);
-
-  priv->quit_icon = gtk_icon_theme_load_icon (theme, "mate-logout", width, 0, NULL);
-
+  gtk_icon_size_lookup (GTK_ICON_SIZE_MENU, &width, &height);
+  priv->quit_icon = gtk_icon_theme_load_icon (theme, "system-log-out", width, 0, NULL);
   priv->button_image = gtk_image_new_from_pixbuf (priv->quit_icon);
+
   gtk_container_add (GTK_CONTAINER (priv->button), priv->button_image);
   gtk_widget_show (priv->button_image);
 
-  gtk_widget_set_tooltip_text (priv->button,
-                               _("Log off, switch user, lock screen or power "
-                                 "down the computer"));
-  gtk_widget_set_tooltip_text (GTK_WIDGET (title), _("Home"));  
-
-  if (priv->show_home_title)
-    gtk_widget_set_state (GTK_WIDGET (title), GTK_STATE_ACTIVE);
-  else
-    gtk_widget_hide (priv->box);
- 
+  gtk_widget_hide (priv->box);
 
   gtk_widget_add_events (GTK_WIDGET (title), GDK_ALL_EVENTS_MASK);
- 
+
   g_signal_connect (priv->screen, "active-window-changed",
                     G_CALLBACK (on_active_window_changed), title);
   g_signal_connect (title, "button-press-event",
@@ -503,10 +640,10 @@ task_title_new (void)
 {
   GtkWidget *title = NULL;
 
-  title = g_object_new (TASK_TYPE_TITLE, 
-                        "border-width", 0, 
+  title = g_object_new (TASK_TYPE_TITLE,
+                        "border-width", 0,
                         "name", "tasklist-button",
-                        "visible-window", FALSE, 
+                        "visible-window", FALSE,
                         NULL);
 
   return title;
